@@ -2,7 +2,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2018. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -76,6 +76,11 @@
 -define(ALTERNATIVE_UNDECODED,alt_undec).
 -define(ALTERNATIVE_PARTS,alt_parts).
 
+%% Removed functions
+
+-removed({decode,'_',"use Mod:decode/2 instead"}).
+-removed({encode,'_',"use Mod:encode/2 instead"}).
+
 %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% This is the interface to the compiler
 
@@ -292,8 +297,7 @@ run_passes_1([{pass,Name,Pass}|Passes], #st{run=Run}=St0)
 	done ->
 	    ok
     catch
-	Class:Error ->
-	    Stk = erlang:get_stacktrace(),
+	Class:Error:Stk ->
 	    io:format("Internal error: ~p:~p\n~p\n",
 		      [Class,Error,Stk]),
 	    {error,{internal_error,{Class,Error}}}
@@ -861,6 +865,7 @@ init_gen_record(EncodingRule, Options) ->
                 _ -> EncodingRule
             end,
     Der = proplists:get_bool(der, Options),
+    Jer = proplists:get_bool(jer, Options) andalso (EncodingRule =/= jer),
     Aligned = EncodingRule =:= per,
     RecPrefix = proplists:get_value(record_name_prefix, Options, ""),
     MacroPrefix = proplists:get_value(macro_name_prefix, Options, ""),
@@ -868,7 +873,7 @@ init_gen_record(EncodingRule, Options) ->
                true -> map;
                false -> record
            end,
-    #gen{erule=Erule,der=Der,aligned=Aligned,
+    #gen{erule=Erule,der=Der,jer=Jer,aligned=Aligned,
          rec_prefix=RecPrefix,macro_prefix=MacroPrefix,
          pack=Pack,options=Options}.
 
@@ -1079,7 +1084,7 @@ get_file_list1(Stream,Dir,Includes,Acc) ->
     end.
 
 get_rule(Options) ->
-    case [Rule || Rule <- [ber,per,uper],
+    case [Rule || Rule <- [ber,per,uper,jer],
 		  Opt <- Options,
 		  Rule =:= Opt] of
 	[Rule] ->
@@ -1689,6 +1694,9 @@ create_pdec_command(ModName,{'CHOICE',[Comp=#'ComponentType'{name=C1}|_]},TNL=[C
     create_pdec_command(ModName,[Comp],TNL,Acc);
 create_pdec_command(ModName,{'CHOICE',[#'ComponentType'{}|Comps]},TNL,Acc) ->
     create_pdec_command(ModName,{'CHOICE',Comps},TNL,Acc);
+create_pdec_command(ModName,{'CHOICE',{Cs1,Cs2}},TNL,Acc)
+  when is_list(Cs1),is_list(Cs2) ->
+    create_pdec_command(ModName,{'CHOICE',Cs1 ++ Cs2},TNL,Acc);
 create_pdec_command(ModName,#'Externaltypereference'{module=M,type=C1},
 		    TypeNameList,Acc) ->
      #type{def=Def} = get_referenced_type(M,C1),
@@ -2390,13 +2398,13 @@ in_process(Fun) ->
     receive
         {Pid, Result}               -> Result;
         {Pid, Class, Reason, Stack} ->
-            ST = try throw(x) catch throw:x -> erlang:get_stacktrace() end,
+            ST = try throw(x) catch throw:x:Stk -> Stk end,
             erlang:raise(Class, Reason, Stack ++ ST)
     end.
 
 process(Parent, Fun) ->
     try
         Parent ! {self(), Fun()}
-    catch Class:Reason ->
-        Parent ! {self(), Class, Reason, erlang:get_stacktrace()}
+    catch Class:Reason:Stack ->
+        Parent ! {self(), Class, Reason, Stack}
     end.
